-- ******************************************************************************************* --
-- * Create Failure Codes\Hierarchies relating to BS ISO 14224 ISO Codes      (MXES Version) * --
-- ******************************************************************************************* --
-- *                                                                                         * --
-- * Implements MRO interpretation of BS ISO 14224 Failure Codes within MAXIMO Environment.  * --
-- *																						 * --
-- * This version for MXES environment (7.1.0)												 * --
-- *                                                                                         * --
-- * This process generates a number of DB2 stored Procedures within the target schema, 	 * --
-- * it then creates additional Failure Codes and Hierarachy entries corresponding to the    * --
-- * ISO 14224 Standard.                                  				 					 * --
-- *                                                                                         * --
-- * Revision History                                                                        * --
-- *                                                                                         * --
-- * Original generated October 2005, based on existing Failure Code Script which used a     * --
-- * series of self referencing select statements to generate the required code sequences,   * --
-- * this version does not use selects against target table, therefore uses auxiliary	     * --
-- * tables and then iterate row by row to insert proper data into Maximo tables. This 		 * --
-- * new design also runs in shorter time period (10 - 15 minutes).                          * --
-- *                                        (version 1.10, 26th February 2008, Denis Rodrigo)* --
-- ******************************************************************************************* --

-- ******************************************************************************************* --
-- * 1) Create Stored Procedures used by Load Process                           	     	 * --
-- ******************************************************************************************* --
/* PROCEDURE Create_Code_Entry */
CREATE PROCEDURE PLUSG_FC_1 ( v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vEnglishLanguageCode VARCHAR(2);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(2000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE v_vFailCode VARCHAR(8);

    DECLARE v_vDesc VARCHAR(100);

    DECLARE cur_1 CURSOR FOR SELECT DISTINCT failurecode, description
                             FROM plusg_fc_aux1
                             WHERE failurecode NOT IN (SELECT failurecode FROM failurecode WHERE orgid = v_pOrgID);

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    WHEN attributename = 'LANGCODE' THEN v_vEnglishLanguageCode
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('FAILURECODE', 'DESCRIPTION', 'FAILURECODEID')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vEnglishLanguageCode = 'EN';

    SET v_vObjectName = 'FAILURECODE';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vFailCode,
                              v_vDesc;

        WHILE l_sqlstatus = 0 DO

            SET v_vSql_stmt = 'INSERT INTO failurecode (failurecode, description, failurecodeid, ' || v_vColumns || ') VALUES (''' || v_vFailCode || ''', ''' || v_vDesc || ''', ' || REPLACE(CHAR(NEXTVAL FOR FAILURECODESEQ), ' ', '') || ', ' || v_vValues || ')';

            EXECUTE IMMEDIATE v_vSql_stmt;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vFailCode,
                                  v_vDesc;

        END WHILE;

        CLOSE cur_1;

END
GO

/* PROCEDURE Create_Remedy_Entry */
CREATE PROCEDURE PLUSG_FC_5 ( v_pFailureListID INTEGER,
                              v_pFailureCode VARCHAR(8),
                              v_pOrgID VARCHAR(8),
                              INOUT v_vSql_stmt VARCHAR(32672) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iFailureListID INTEGER;

    DECLARE v_vType VARCHAR(8);

    DECLARE v_vFailCode VARCHAR(12);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE cur_1 CURSOR FOR SELECT failurecode
                             FROM plusg_fc_aux1
                             WHERE type = v_vType;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'TYPE' THEN v_vType
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('FAILURELIST', 'FAILURECODE', 'PARENT')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vType = 'REMEDY';

    SET v_vObjectName = 'FAILURELIST';

    SET v_vColumns = '';

    SET v_vValues = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vFailCode;

        WHILE l_sqlstatus = 0 DO

            IF SUBSTR(v_vSql_stmt, 1, 6) = 'INSERT' THEN
                SET v_vSql_stmt = v_vSql_stmt || ', (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || REPLACE(CHAR(v_pFailureListID), ' ', '') || ', ' || v_vValues || ')';
                IF LENGTH(v_vSql_stmt) > 32200 THEN
                   EXECUTE IMMEDIATE v_vSql_stmt;
                   SET v_vSql_stmt = '';
                END IF;
            ELSE
                SET v_vSql_stmt = 'INSERT INTO failurelist (failurelist, failurecode, parent, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || REPLACE(CHAR(v_pFailureListID), ' ', '') || ', ' || v_vValues || ')';
            END IF;

            VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vFailCode;

        END WHILE;

        CLOSE cur_1;

END
GO

/* PROCEDURE Create_Cause_Entry */
CREATE PROCEDURE PLUSG_FC_4 ( v_pFailureListID INTEGER,
                              v_pFailureCode VARCHAR(8),
                              v_pOrgID VARCHAR(8),
                              INOUT v_vSql_stmt VARCHAR(32672) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iFailureListID INTEGER;

    DECLARE v_vType VARCHAR(8);

    DECLARE v_vFailCode VARCHAR(12);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE cur_1 CURSOR FOR SELECT failurecode
                             FROM plusg_fc_aux1
                             WHERE type = v_vType;

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'TYPE' THEN v_vType
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('FAILURELIST', 'FAILURECODE', 'PARENT')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vType = 'CAUSE';

    SET v_vObjectName = 'FAILURELIST';

    SET v_vColumns = '';

    SET v_vValues = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vFailCode;

        WHILE l_sqlstatus = 0 DO

            IF SUBSTR(v_vSql_stmt, 1, 6) = 'INSERT' THEN
                SET v_vSql_stmt = v_vSql_stmt || ', (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || REPLACE(CHAR(v_pFailureListID), ' ', '') || ', ' || v_vValues || ')';
                IF LENGTH(v_vSql_stmt) > 32200 THEN
                   EXECUTE IMMEDIATE v_vSql_stmt;
                   SET v_vSql_stmt = '';
                END IF;
            ELSE
                SET v_vSql_stmt = 'INSERT INTO failurelist (failurelist, failurecode, parent, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || REPLACE(CHAR(v_pFailureListID), ' ', '') || ', ' || v_vValues || ')';
            END IF;

            CALL PLUSG_FC_5 (v_iFailureListID, v_vFailCode, v_pOrgID, v_vSql_stmt);

			VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vFailCode;

        END WHILE;

        CLOSE cur_1;

END
GO

/* PROCEDURE Create_Problem_Entry */
CREATE PROCEDURE PLUSG_FC_3 ( v_pFailureListID INTEGER,
                              v_pFailureCode VARCHAR(8),
                              v_pOrgID VARCHAR(8),
                              INOUT v_vSql_stmt VARCHAR(32672) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iFailureListID INTEGER;

    DECLARE v_vType VARCHAR(8);

    DECLARE v_vFailCode VARCHAR(12);

    DECLARE v_vAbbrevFailCode VARCHAR(8);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE l_sqlstatus INTEGER;

    DECLARE cur_1 CURSOR FOR SELECT failurecode, abbrevfailcode
                             FROM plusg_fc_aux1
                             WHERE type = v_vType
                             AND abbrevfailcode = SUBSTR(v_pFailureCode,1,4) OR abbrevfailcode = SUBSTR(v_pFailureCode,1,2);

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'TYPE' THEN v_vType
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('FAILURELIST', 'FAILURECODE', 'PARENT')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vType = 'PROBLEM';

    SET v_vObjectName = 'FAILURELIST';

    SET v_vColumns = '';

    SET v_vValues = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID; 

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vFailCode, v_vAbbrevFailCode;

        WHILE l_sqlstatus = 0 DO

            IF SUBSTR(v_vSql_stmt, 1, 6) = 'INSERT' THEN
                SET v_vSql_stmt = v_vSql_stmt || ', (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || REPLACE(CHAR(v_pFailureListID), ' ', '') || ', ' || v_vValues || ')';
                IF LENGTH(v_vSql_stmt) > 32200 THEN
                   EXECUTE IMMEDIATE v_vSql_stmt;
                   SET v_vSql_stmt = '';
                END IF;
            ELSE
                SET v_vSql_stmt = 'INSERT INTO failurelist (failurelist, failurecode, parent, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || REPLACE(CHAR(v_pFailureListID), ' ', '') || ', ' || v_vValues || ')';
            END IF;

            CALL PLUSG_FC_4 (v_iFailureListID, v_vFailCode, v_pOrgID, v_vSql_stmt);

			VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID; 

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vFailCode,
                                  v_vAbbrevFailCode;

        END WHILE;

        CLOSE cur_1;

END
GO

/* PROCEDURE Create_Hierarchy_Entry */
CREATE PROCEDURE PLUSG_FC_2 ( v_pOrgID VARCHAR(8) )
LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_iFailureListID INTEGER;

    DECLARE v_vType VARCHAR(8);

    DECLARE v_vFailCode VARCHAR(12);

    DECLARE v_vAttributeName VARCHAR(100);

    DECLARE v_vDefaultValue VARCHAR(256);

    DECLARE v_vRequired INTEGER;

    DECLARE v_vMaxType VARCHAR(8);

    DECLARE v_vObjectName VARCHAR(100);

    DECLARE v_vColumns VARCHAR(1000);

    DECLARE v_vValues VARCHAR(1000);

    DECLARE v_vSql_stmt VARCHAR(32672);

    DECLARE l_sqlstatus INTEGER;

    DECLARE cur_1 CURSOR FOR SELECT failurecode
                             FROM plusg_fc_aux1
                             WHERE type IS NULL
                             AND failurecode NOT IN (SELECT failurecode FROM failurelist WHERE parent IS NULL and orgid = v_pOrgID);

    DECLARE cur_0 CURSOR FOR SELECT attributename,
                                    required,
                                    maxtype,
                                    CASE
                                    WHEN attributename = 'TYPE' THEN v_vType
                                    WHEN attributename = 'ORGID' THEN v_pOrgID
                                    ELSE defaultvalue
                                    END AS defaultvalue
                             FROM maxattribute
                             WHERE objectname = v_vObjectName
                             AND persistent = 1
                             AND attributename NOT IN ('FAILURELIST', 'FAILURECODE')
                             ORDER BY attributeno;

    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_sqlstatus = -1;

    DECLARE CONTINUE HANDLER FOR SQLWARNING

        SET l_error = SQLSTATE;

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION

        BEGIN

            SET l_error = SQLSTATE;
            SET l_sqlstatus = -1;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vObjectName = 'FAILURELIST';

    SET v_vColumns = '';

    SET v_vValues = '';

    SET v_vSql_stmt = '';

        OPEN cur_0;

        SET l_sqlstatus = 0;
        FETCH FROM cur_0 INTO v_vAttributeName,
                              v_vRequired,
                              v_vMaxType,
                              v_vDefaultValue;

        WHILE l_sqlstatus = 0 DO

            IF v_vColumns = '' THEN

                SET v_vColumns = v_vColumns || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || 'null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

	                            SET v_vValues = v_vValues || '''' || v_vDefaultValue || '''';
	
	                        ELSE
	
	                        	SET v_vValues = v_vValues || v_vDefaultValue;
	
	                        END IF;

                        END IF;

                    END IF;

                END IF;

            ELSE

                SET v_vColumns = v_vColumns || ',' || v_vAttributeName;

                IF (v_vDefaultValue IS NULL)
                   AND (v_vRequired = 1) THEN

                    SET v_vValues = v_vValues || ',' || '0';

                ELSE

                    IF v_vDefaultValue IS NULL THEN

                        SET v_vValues = v_vValues || ', null';

                    ELSE

                        IF UPPER(REPLACE(v_vDefaultValue, '&', '')) = 'SYSDATE' THEN

                            SET v_vValues = v_vValues || ',' || CHAR(CURRENT TIMESTAMP);

                        ELSE

                        	IF (v_vMaxType != 'INTEGER') AND (v_vMaxType != 'YORN') AND (v_vMaxType != 'SMALLINT') THEN

                            	SET v_vValues = v_vValues || ',''' || v_vDefaultValue || '''';
                            	
                           	ELSE
                           	
                           		SET v_vValues = v_vValues || ', ' || v_vDefaultValue;
                           	
                           	END IF;

                        END IF;

                    END IF;

                END IF;

            END IF;

            SET l_sqlstatus = 0;
            FETCH FROM cur_0 INTO v_vAttributeName,
                                  v_vRequired,
                                  v_vMaxType,
                                  v_vDefaultValue;

        END WHILE ;

        CLOSE cur_0;

        OPEN cur_1;

        VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID;

        SET l_sqlstatus = 0;
        FETCH FROM cur_1 INTO v_vFailCode;

        WHILE l_sqlstatus = 0 DO

            IF SUBSTR(v_vSql_stmt, 1, 6) = 'INSERT' THEN
                SET v_vSql_stmt = v_vSql_stmt || ', (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || v_vValues || ')';
                IF LENGTH(v_vSql_stmt) > 32200 THEN
                   EXECUTE IMMEDIATE v_vSql_stmt;
                   SET v_vSql_stmt = '';
                END IF;
            ELSE
                SET v_vSql_stmt = 'INSERT INTO failurelist (failurelist, failurecode, ' || v_vColumns || ') VALUES (' || REPLACE(CHAR(v_iFailureListID), ' ', '') || ', ''' || v_vFailCode || ''', ' || v_vValues || ')';
            END IF;

            CALL PLUSG_FC_3 (v_iFailureListID, v_vFailCode, v_pOrgID, v_vSql_stmt);

			VALUES NEXTVAL FOR FAILURELISTSEQ INTO v_iFailureListID;

            SET l_sqlstatus = 0;
            FETCH FROM cur_1 INTO v_vFailCode;

        END WHILE;

        IF v_vSql_stmt != '' THEN

            EXECUTE IMMEDIATE v_vSql_stmt;

        END IF;

        CLOSE cur_1;

END
GO

/* PROCEDURE Insert_Failure_Codes */
CREATE PROCEDURE PlusG_FC_6 (v_pOrgID VARCHAR(8) )

LANGUAGE SQL

BEGIN

    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';

    DECLARE l_error CHAR(5) DEFAULT '00000';

    DECLARE v_vOrgID_Exists VARCHAR(1);
    
    DECLARE CONTINUE HANDLER FOR NOT FOUND

        SET l_error = '00000';

    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION, SQLWARNING

        BEGIN

            SET l_error = SQLSTATE;

            IF SUBSTR(l_error, 1, 1) >= '5'
               AND SUBSTR(l_error, 1, 1) <= '9' THEN

                RESIGNAL;

            END IF;

        END;

    SET v_vOrgID_Exists = 'N';
    
    SELECT COALESCE(MAX('Y'), 'N')
    INTO v_vOrgID_Exists
    FROM organization
    WHERE organization.orgid = v_pOrgID
    FETCH FIRST 1 ROWS ONLY;
    
    IF v_vOrgID_Exists = 'Y' THEN
    	CALL PLUSG_FC_1 (v_pOrgID);
    	CALL PLUSG_FC_2 (v_pOrgID);
    END IF;

END
GO

-- ******************************************************************************************* --
-- * End of Create Failure Codes\Hierarchies relating to BS ISO 14224...      (MXES Version) * --
-- ******************************************************************************************* --               